import { h, Host } from "@stencil/core";
import { UAParser } from "ua-parser-js";
import { v4 as uuidv4 } from "uuid";
import { Environment, ErrorCorrectionLevel, ThemePreset } from "./enums";
import { Logger } from "../../utils/utils";
import * as utilsURL from "../../utils/url";
import Logo from "./logo";
import { WS } from "./websocket";
const maxInterval = 59 * 60 * 1000;
const minInterval = 10 * 1000;
const DEFAULT_CALLBACK_URL = window.location.origin + '/zero/auth';
const ANIMATION_DURATION = 3000;
export class ZeroAccount {
    constructor() {
        this.refreshQR = (ignoreAnimate = false) => {
            this.initServices();
            const update = () => {
                this.restartTimer();
                this.updateURL();
            };
            if (ignoreAnimate)
                return update();
            this.isAnimating = true;
            this.animationTimer = window.setTimeout(() => {
                update();
                this.isAnimating = false;
            }, ANIMATION_DURATION);
        };
        this.hideOnEsc = (e) => {
            if ((e.key === 'Escape' || e.key === 'Esc')) {
                this.modalOpen = false;
            }
        };
        this.log = (...args) => this.logger.log(...args);
        this.appId = undefined;
        this.updateInterval = 3 * 60 * 1000;
        this.callbackURL = DEFAULT_CALLBACK_URL;
        this.level = ErrorCorrectionLevel.M;
        this.environment = Environment.production;
        this.enableWithCredentials = false;
        this.themePreset = ThemePreset.light;
        this.autoStartApp = true;
        this.modalOpen = false;
        this.isOpenAppBeingClicked = false;
        this.isAnimating = false;
        this.url = '';
    }
    getUpdateInterval() {
        // make sure the provided value is within bounds
        return Math.min(Math.max(this.updateInterval, minInterval), maxInterval);
    }
    toggleModal() {
        this.modalOpen = !this.modalOpen;
        if (this.modalOpen)
            this.refreshQR(true);
        if ((this.modalOpen && this.autoStartApp) || this.isMobile())
            this.openApp();
    }
    _closeModal() {
        this.modalOpen = false;
        this.stopTimer();
    }
    logout() {
        this.ws.logout();
    }
    closeModal(e) {
        if (e.target !== this.modalEl && e.target !== this.closeButtonEl) {
            return e.stopPropagation();
        }
        this._closeModal();
    }
    openApp() {
        this.refreshQR(true);
        // TODO: uncomment if the fix with isOpenAppClicked does not work
        // const confirmBeforeClose = (event) => {
        //   event = event || window.event;
        //   if (!event) return;
        //   event.preventDefault();
        //   event.returnValue = '';
        //   delete event['returnValue'];
        //   return '';
        // }
        //
        // window.addEventListener('beforeunload', confirmBeforeClose);
        if (this.isOpenAppBeingClicked)
            return;
        this.isOpenAppBeingClicked = true;
        window.location.href = this.url;
        setTimeout(() => {
            // window.removeEventListener('beforeunload', confirmBeforeClose);
            this.isOpenAppBeingClicked = false;
        }, 300);
    }
    render() {
        const logoColor = this.themePreset === ThemePreset.light ? '#7263FA' : '#FFFFF9';
        const qrColor = '#000';
        return (h(Host, { key: 'fdf5ca45b68d576ba5b71d0f48b272b344caeca8' }, h("slot", { key: '38e565a3532aa76ab55d35a6a12a6b46ffb57510' }), this.host.childNodes.length === 0 &&
            h("button", { key: 'db9461d518bd2f9d304d88884c326128f3e71f60', class: `button ${this.themePreset} ${this.isOpenAppBeingClicked ? 'disabled' : ''}`, onClick: () => this.toggleModal() }, h(Logo, { key: '18a4d7bfc11e1c9e633854f603357039781b7de3', className: 'button__logo', color: logoColor }), "use 0account"), h("div", { key: '5c3f167aa8a400c019a92160f2b6a9bb9d4bd31f', ref: el => this.modalEl = el, onClick: (e) => this.closeModal(e), class: `modal ${this.themePreset} ${this.isAnimating ? 'modal-animated' : ''} ${this.modalOpen ? '' : 'modal--hidden'}` }, h("div", { key: 'c1fc1c117c54a0085404e6b22238e201e653fbb8', class: 'modal__content' }, h("div", { key: '59b094c5105aaf55cb2db4888f88dbc73c99cd67', ref: el => this.closeButtonEl = el, class: 'modal__close-button' }), h("div", { key: '3171c6d422b95fb2c6e452f8f78c59103b00eeae', class: 'modal__qrcode__container' }, h("qr-code", { key: '83874926b0e682e103578e5677585fcc7af5b14b', url: this.url, class: "modal__qrcode", level: this.level, color: qrColor }), h("div", { key: '830bb649067b48654ad18f12ce85ab9d4c179a73', class: 'circle__overlay' }, h("div", { key: '4e41c864cf858e942cf362b77b420c5725e93250', class: 'circle' }), h("div", { key: 'bf7c57f92501286059118dfe7ffd7e15161fbd89', class: 'circle circle--medium' }), h("div", { key: 'a5f45b506b93571a7cb958adaa8533f15d97c8ae', class: 'circle circle--large' })), h(Logo, { key: '1d1710cdc1c89c0ec75960b4d83f1f717abf876b', className: 'modal__logo', color: logoColor })), h("div", { key: 'a6c223ab3d791b73f448d6415f8efb100dbebe05', class: 'modal__buttons' }, h("button", { key: '6c4eff22956ebe3825441399558fa1f7b8337fa6', disabled: this.isAnimating || this.isOpenAppBeingClicked, onClick: () => this.openApp(), class: 'modal__button' }, "Open 0account"), h("a", { key: '4236ece1098075d20af7f52c82ff41ac2ced5eb1', href: 'https://0account.com/downloads', class: 'modal__link' }, "Download"))))));
    }
    initServices() {
        if (this.logger && this.uaParser && this.ws)
            return;
        this.logger = new Logger(this.environment);
        this.uaParser = new UAParser();
        this.ws = new WS(this.appId, {
            updateInterval: this.getUpdateInterval(),
            enableWithCredentials: this.enableWithCredentials,
            callbackURL: this.callbackURL,
            uaParser: this.uaParser,
            env: this.environment,
            log: this.log,
        });
    }
    // called only once
    componentWillLoad() {
        this.initServices();
        // make sure we initQR
        this.refreshQR(true);
        document.addEventListener('keydown', this.hideOnEsc);
    }
    updateURL() {
        const expires = new Date().getTime() + this.getUpdateInterval();
        const uuid = uuidv4();
        this.ws.updateUUID(uuid);
        const params = utilsURL.appendToParams('', {
            appId: this.appId,
            uuid: uuid,
            expires: expires,
            domain: window.location.hostname,
        });
        if (this.isMobile()) {
            this.url = 'https://launch.0account.com/auth?' + params;
        }
        else {
            this.url = 'zero-account://auth?' + params;
        }
    }
    isMobile() {
        const device = this.uaParser.getDevice();
        return device.type === 'tablet' || device.type === 'mobile';
    }
    connectedCallback() {
        this.startTimer();
    }
    disconnectedCallback() {
        this.stopTimer();
        document.removeEventListener('keydown', this.hideOnEsc);
    }
    restartTimer() {
        this.stopTimer();
        this.startTimer();
    }
    startTimer() {
        this.timer = window.setInterval(this.refreshQR, this.getUpdateInterval());
    }
    stopTimer() {
        window.clearInterval(this.timer);
        window.clearTimeout(this.animationTimer);
        this.isAnimating = false;
    }
    static get is() { return "zero-account"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["zero-account.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["zero-account.css"]
        };
    }
    static get properties() {
        return {
            "appId": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": true,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "app-id",
                "reflect": false
            },
            "updateInterval": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "update-interval",
                "reflect": false,
                "defaultValue": "3 * 60 * 1000"
            },
            "callbackURL": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "callback-url",
                "reflect": false,
                "defaultValue": "DEFAULT_CALLBACK_URL"
            },
            "level": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ErrorCorrectionLevel",
                    "resolved": "ErrorCorrectionLevel.H | ErrorCorrectionLevel.L | ErrorCorrectionLevel.M | ErrorCorrectionLevel.Q",
                    "references": {
                        "ErrorCorrectionLevel": {
                            "location": "import",
                            "path": "./enums",
                            "id": "src/components/zero-account/enums.ts::ErrorCorrectionLevel"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "level",
                "reflect": false,
                "defaultValue": "ErrorCorrectionLevel.M"
            },
            "environment": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "Environment",
                    "resolved": "Environment.production | Environment.staging",
                    "references": {
                        "Environment": {
                            "location": "import",
                            "path": "./enums",
                            "id": "src/components/zero-account/enums.ts::Environment"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "environment",
                "reflect": false,
                "defaultValue": "Environment.production"
            },
            "enableWithCredentials": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "enable-with-credentials",
                "reflect": false,
                "defaultValue": "false"
            },
            "themePreset": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ThemePreset",
                    "resolved": "ThemePreset.dark | ThemePreset.light",
                    "references": {
                        "ThemePreset": {
                            "location": "import",
                            "path": "./enums",
                            "id": "src/components/zero-account/enums.ts::ThemePreset"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "theme-preset",
                "reflect": false,
                "defaultValue": "ThemePreset.light"
            },
            "autoStartApp": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "auto-start-app",
                "reflect": false,
                "defaultValue": "true"
            }
        };
    }
    static get states() {
        return {
            "modalOpen": {},
            "isOpenAppBeingClicked": {},
            "isAnimating": {},
            "url": {}
        };
    }
    static get elementRef() { return "host"; }
    static get listeners() {
        return [{
                "name": "0account-toggle-modal",
                "method": "toggleModal",
                "target": "document",
                "capture": false,
                "passive": false
            }, {
                "name": "0account-close-modal",
                "method": "_closeModal",
                "target": "document",
                "capture": false,
                "passive": false
            }, {
                "name": "0account-authenticated",
                "method": "_closeModal",
                "target": "document",
                "capture": false,
                "passive": false
            }, {
                "name": "0account-logout",
                "method": "logout",
                "target": "document",
                "capture": false,
                "passive": false
            }, {
                "name": "0account-open-app",
                "method": "openApp",
                "target": "document",
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=zero-account.js.map
